/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package generator;

import InputOutput.ReverseWrapper;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.IllegalFormatException;
import java.util.Scanner;
import java.util.logging.Level;
import java.util.logging.Logger;
import structures.Attribute;
import structures.IAttribute;
import structures.IReference;
import structures.ITable;
import structures.Reference;
import structures.SqlDataType;
import structures.Table;

/**
 *
 * @author mates
 */
public class DefaultGenerator implements IGenerator {

    private ArrayList<ITable> tables;
    private ArrayList<IReference> references;

    public DefaultGenerator(ArrayList<ITable> tables, ArrayList<IReference> refs) {
        this.tables = tables;
        this.references = refs;
    }

    public DefaultGenerator() {
    }

    public String generate() {
        String NL = System.getProperty("line.separator");
        StringBuilder s = new StringBuilder();
        for (int i = 0; i < tables.size(); i++) {
            s.append("CREATE TABLE ").append(tables.get(i).getName()).append(" (").append(NL);
            for (int j = 0; j < tables.get(i).getAttributeList().size(); j++) {
                IAttribute attr = tables.get(i).getAttributeList().get(j);
                s.append("\t\t").append(attr.getName());
                s.append("\t\t").append(attr.getDataType().getName());
                if (!attr.getDataType().getArgs().isEmpty()) {
                    s.append("( ");
                    for (int k = 0; k < attr.getDataType().getArgs().size(); k++) {
                        s.append(attr.getArguments().get(i).intValue());
                        if (k != attr.getDataType().getArgs().size() - 1) {
                            s.append(", ");
                        }
                    }
                    s.append(" )");
                }
                if (attr.isNotNull()) {
                    s.append("\t\tNOT NULL");
                }
                if (j != tables.get(i).getAttributeList().size() - 1) {
                    s.append(",").append(NL);
                }
            }
            s.append(NL).append(");");
            s.append(NL).append(NL);
        }
        for (int i = 0; i < tables.size(); i++) {
            if (tables.get(i).getPrimaryKey().size() == 0) {
                continue;
            }
            s.append("ALTER TABLE ").append(tables.get(i).getName()).append(NL);
            s.append("\tADD CONSTRAINT ").append(tables.get(i).getName()).append("_PK ");
            s.append("PRIMARY KEY ( ");
            for (int j = 0; j < tables.get(i).getPrimaryKey().size(); j++) {
                s.append(tables.get(i).getPrimaryKey().get(j).getName());
                if (tables.get(i).getPrimaryKey().size() - 1 != j) {
                    s.append(", ");
                }
            }
            s.append(" );").append(NL).append(NL).append(NL);
        }

        for (int i = 0; i < references.size(); i++) {
            ITable tpk = references.get(i).getTableWithFK();
            ITable rt = references.get(i).getReferencedTable();
            s.append("ALTER TABLE ").append(references.get(i).getTableWithFK().getName()).append(NL);
            s.append("\tADD CONSTRAINT ").append(tpk.getName()).append("_").append(rt.getName()).append("_FK ");
            s.append("FOREIGN KEY ( ");
            for (int j = 0; j < references.get(i).getAttributeWithFK().size(); j++) {
                IAttribute attr = references.get(i).getAttributeWithFK().get(j);
                s.append(attr.getName());
                if (references.get(i).getAttributeWithFK().size() - 1 != j) {
                    s.append(", ");
                }
            }
            s.append(" ) REFERENCES ");
            s.append(references.get(i).getReferencedTable().getName());
            s.append(" ( ");
            for (int j = 0; j < references.get(i).getReferencedTable().getPrimaryKey().size(); j++) {
                IAttribute attr = references.get(i).getReferencedTable().getPrimaryKey().get(j);
                s.append(attr.getName());
                if (references.get(i).getReferencedTable().getPrimaryKey().size() - 1 != j) {
                    s.append(", ");
                }
            }
            s.append(" );").append(NL).append(NL).append(NL);
        }

        return s.toString();
    }

    public ReverseWrapper reverseEngineer(Scanner scanner) throws IllegalFormatException {
        ArrayList<ITable> tabs = new ArrayList<ITable>();
        ArrayList<IReference> refs = new ArrayList<IReference>();
        ArrayList<String> actions = new ArrayList<String>();
        StringBuilder source = new StringBuilder();
        String action;

        String NL = System.getProperty("line.separator");
        try {
            while (scanner.hasNextLine()) {
                String s = scanner.nextLine();
                if (s.trim().indexOf("--") != -1) {
                    s = s.substring(0, s.indexOf("--"));
                }
                source.append(s).append(NL);
            }
        } finally {
            scanner.close();
        }

        actions.addAll(Arrays.asList(source.toString().split(";")));
        for (int i = 0; i < actions.size(); i++) {
            actions.set(i, actions.get(i).trim());
//            System.out.println("***/"+actions.get(i)+"/***");
        }

        for (String string : actions) {
            if (string.contains("CREATE TABLE")) {
//                // creating new table
                action = new String(string);
                String tableName = string.substring(string.indexOf("CREATE TABLE") + 12, string.indexOf("(")).trim();
                tabs.add(new Table(tableName));
//                  System.out.println("table name: /"+tableName+"/");
                action = action.substring(action.indexOf("(") + 1, action.lastIndexOf(")")).trim();
//                  System.out.println("action: /"+action+"/");
                ArrayList<String> attributes = new ArrayList<String>();
                attributes.addAll(Arrays.asList(action.toString().split(",")));
                for (int i = 0; i < attributes.size(); i++) {
                    attributes.set(i, attributes.get(i).trim());
                    // System.out.println("Atribut: " + attributes.get(i));
                    // zjistim info o atributu a mám v podstatě tvorbu tabulky v pohodě ;-)
                    int firstWhiteSpace = 0;
                    String attr = new String(attributes.get(i));
                    while (!Character.isWhitespace(attr.charAt(firstWhiteSpace))) {
                        firstWhiteSpace++;
                    }
                    String attrName = attr.substring(0, firstWhiteSpace);
                    // System.out.println("Atribut: /" + attrName+ "/");
                    attr = attr.substring(firstWhiteSpace).trim();

                    firstWhiteSpace = 0;
                    while (!Character.isWhitespace(attr.charAt(firstWhiteSpace))) {
                        firstWhiteSpace++;
                        if (firstWhiteSpace >= attr.length()) {
                            break;
                        }
                    }
                    boolean b = false;
                    while (firstWhiteSpace < attr.length() && b == false) {
                        b = true;
                        if (attr.substring(firstWhiteSpace).contains("CHARACTER")) {
                            b = false;
                            firstWhiteSpace = attr.indexOf("CHARACTER") + 9;
                        }
                        if (attr.substring(firstWhiteSpace).contains("VARYING")) {
                            b = false;
                            firstWhiteSpace = attr.indexOf("VARYING") + 7;
                        }
                        if (attr.substring(firstWhiteSpace).contains("PRECISION")) {
                            b = false;
                            firstWhiteSpace = attr.indexOf("PRECISION") + 9;
                        }
                    }

                    String dataType = new String(attr.substring(0, firstWhiteSpace).trim());
                    boolean notNull = attr.contains("NOT NULL");
                    boolean unique = attr.contains("UNIQUE");

                    String args = "";
                    if (firstWhiteSpace < attr.length()) {
                        if (attr.indexOf(")") != -1) {
                            firstWhiteSpace = attr.indexOf(")") + 1;
                            args = attr.substring(attr.indexOf("("), attr.indexOf(")") + 1);
                            attr = attr.substring(0, attr.indexOf("("));
                        }
                    }

                    //System.out.println("Atribut: /" + attrName+ "/ data type: /"+dataType+"/ arguments: /"+args+"/");

                    ArrayList<String> argList = new ArrayList<String>();
                    if (!args.equals("")) {
                        args = args.substring(1, args.length() - 1).trim();
                        argList.addAll(Arrays.asList(args.split(",")));
                        for (int j = 0; j < argList.size(); j++) {
                            argList.set(j, argList.get(j).trim());
                        }
                    }


                    Attribute attribute = new Attribute(
                            attrName,
                            new SqlDataType(dataType, dataType, false, argList));
                    attribute.setNotNull(notNull);
                    attribute.setUnique(unique);
                    tabs.get(tabs.size() - 1).getAttributeList().add(attribute);
                }

            }
            if (string.contains("ALTER TABLE")) {
                action = string;
                String rest = action.substring(string.indexOf("ALTER TABLE") + 11).trim();
                //       System.out.println(rest);
                int firstWhiteSpace = 0;
                while (!Character.isWhitespace(rest.charAt(firstWhiteSpace))) {
                    firstWhiteSpace++;
                    if (firstWhiteSpace >= rest.length()) {
                        break;
                    }
                }
                String tableName = rest.substring(0, firstWhiteSpace).trim();
                //         System.out.println("/" + tableName + "/");
                ITable referenceToTable = null;
                for (ITable t : tabs) {
                    if (t.getName().equals(tableName)) {
                        referenceToTable = t;
                        break;
                    }
                }
                if (referenceToTable == null) {
                    continue;
                }

                rest = rest.substring(firstWhiteSpace).trim();
                ArrayList<String> alterActions = new ArrayList<String>();
                alterActions.addAll(Arrays.asList(rest.split("ADD CONSTRAINT")));
                for (String alterAction : alterActions) {
                    if (alterAction.contains("PRIMARY KEY")) {
                        String keyAttrs = alterAction.substring(alterAction.indexOf("(") + 1, alterAction.indexOf(")")).trim();
                        //  System.out.println(keyAttrs);
                        ArrayList<String> keyAttrsArray = new ArrayList<String>();
                        keyAttrsArray.addAll(Arrays.asList(keyAttrs.split(",")));
                        for (int i = 0; i < keyAttrsArray.size(); i++) {
                            keyAttrsArray.set(i, keyAttrsArray.get(i).trim());
                            IAttribute pk = null;
                            for (IAttribute attr : referenceToTable.getAttributeList()) {
                                if (attr.getName().compareTo(keyAttrsArray.get(i)) == 0) {
                                    attr.setPrimaryKey(true);
                                }
                            }
                        }
                    } else if (alterAction.contains("FOREIGN KEY")) {
                        String fkAttributes = alterAction.substring(
                                alterAction.indexOf("FOREIGN KEY") + 11,
                                alterAction.indexOf("REFERENCES")).trim();
                        fkAttributes = fkAttributes.substring(1, fkAttributes.length() - 1).trim();
                        ArrayList<String> fkAttrsArray = new ArrayList<String>();
                        ArrayList<IAttribute> fkAttrRefs = new ArrayList<IAttribute>();
                        fkAttrsArray.addAll(Arrays.asList(fkAttributes.split(",")));
                        for (int i = 0; i < fkAttrsArray.size(); i++) {
                            fkAttrsArray.set(i, fkAttrsArray.get(i).trim());
                            //                    System.out.println(fkAttrsArray.get(i));
                            for (IAttribute attr : referenceToTable.getAttributeList()) {
                                if (attr.getName().compareTo(fkAttrsArray.get(i)) == 0) {
                                    fkAttrRefs.add(attr);
                                }
                            }
                        }
                        String referencedTableName = alterAction.substring(alterAction.indexOf("REFERENCES") + 10).trim();
                        int firstWhiteChar = 0;
                        while (!Character.isWhitespace(referencedTableName.charAt(firstWhiteChar))) {
                            firstWhiteChar++;
                            if (firstWhiteSpace >= referencedTableName.length()) {
                                break;
                            }
                        }
                        referencedTableName = referencedTableName.substring(0, firstWhiteChar).trim();
                        //                System.out.println(referencedTableName);
                        ITable referencedTable = null;
                        for (ITable t : tabs) {
                            if (t.getName().equals(referencedTableName)) {
                                referencedTable = t;
                                break;
                            }
                        }
                        if (referencedTable == null) {
                            continue;
                        }
                        if (fkAttrRefs.size() == referencedTable.getPrimaryKey().size()) {
                            refs.add(new Reference(
                                    referenceToTable,
                                    fkAttrRefs,
                                    referencedTable));
                        }
                    }
                }
            }
        }

        ReverseWrapper rw = new ReverseWrapper(tabs, refs);
        return rw;
    }

    public ArrayList<IReference> getReferences() {
        return references;
    }

    public ArrayList<ITable> getTables() {
        return tables;
    }

    public void setReferences(ArrayList<IReference> references) {
        this.references = references;
    }

    public void setTables(ArrayList<ITable> tables) {
        this.tables = tables;
    }

    public static void main(String[] args) {
        DefaultGenerator generator = new DefaultGenerator();
        try {
            generator.reverseEngineer(new Scanner(new FileInputStream(new File("skript.sql"))));
        } catch (FileNotFoundException ex) {
            Logger.getLogger(DefaultGenerator.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
}
