/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package graphics;

import init.DefaultDataTypes;
import init.IDataTypes;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Point;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseMotionAdapter;
import java.awt.event.MouseMotionListener;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.Observable;
import javax.swing.ImageIcon;
import javax.swing.JComponent;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JTextField;
import javax.swing.border.LineBorder;
import structures.Attribute;
import structures.IAttribute;
import structures.ISqlDataType;
import structures.ITable;

/**
 *
 * @author mates
 */
public class TableComponent extends JComponent implements Serializable {

    private ITable table;
    private boolean active = true;
    private int scalingMode = 0;
    private Observ observ = new Observ();
    private Point lastMousePressedPoint = new Point();
    private ImageIcon rightDownImg = new javax.swing.ImageIcon(getClass().getResource("/resources/right_down.png"));
    private JLabel rightDownLabel = new JLabel(rightDownImg);
    private JLabel tableName = new JLabel();
    private JTextField textFieldTableName = new JTextField();
    private int attrSequence = 1;
    private DefaultDataTypes defaultTypes;
    ArrayList<IDataTypes> dbSpecificTypes;
    private ImageIcon icons[] = new ImageIcon[5];

    public TableComponent(ITable table, PanelModel panelModel, MainFrame mainFrame, DefaultDataTypes dTypes, ArrayList<IDataTypes> dbSpecificTypes) {
        this.table = table;
        this.defaultTypes = dTypes;
        this.dbSpecificTypes = dbSpecificTypes;

        observ.addObserver(panelModel);
        observ.addObserver(mainFrame);

        addMouseListener(returnTableComponentMouseListener());
        addMouseMotionListener(returnTableComponentMouseMotionListener());

        icons[0] = new ImageIcon(getClass().getResource("/resources/auto_increment.png"));
        icons[1] = new ImageIcon(getClass().getResource("/resources/not_null.png"));
        icons[2] = new ImageIcon(getClass().getResource("/resources/unique.png"));
        icons[3] = new ImageIcon(getClass().getResource("/resources/key.png"));
        icons[4] = new ImageIcon(getClass().getResource("/resources/foreign_key.png"));

        this.add(rightDownLabel);
        rightDownLabel.setBounds(229, 99, 21, 21);
        rightDownLabel.addMouseListener(returnRightDownMouseListener());
        rightDownLabel.addMouseMotionListener(returnRightDownMouseMotionListener());

        this.add(tableName);
        tableName.setText(table.getName());
        tableName.setBounds(5, 0, 145, 20);
        this.add(textFieldTableName);
        textFieldTableName.setBounds(5, 0, 145, 20);
        textFieldTableName.setVisible(false);
        textFieldTableName.addKeyListener(returnTextBoxKeyListener());

        observ.notifyObservers(this);

        this.setComponentZOrder(rightDownLabel, 0);

    }

    public ITable getTable() {
        return table;
    }

    public void setTable(ITable table) {
        this.table = table;
    }

    public boolean isActive() {
        return active;
    }

    public void setActive(boolean active) {
        this.active = active;
        rightDownLabel.setVisible(active);
        if (textFieldTableName.isVisible()) {
            setTableNameByTextBox();
        }
        tableName.setVisible(true);
        textFieldTableName.setVisible(false);
    }

    private void setTableNameByTextBox() {
        if (textFieldTableName.getText().contains(" ")) {
            return;
        }
        tableName.setText(textFieldTableName.getText());
        table.setName(textFieldTableName.getText());
    }

    public JLabel getRightDownLabel() {
        return rightDownLabel;
    }

    @Override
    public Dimension getPreferredSize() {
        return new Dimension(250, 120);
    }

    @Override
    protected void paintComponent(Graphics g) {
        super.paintComponent(g);

        g.setColor(Color.white);
        g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
        if (active) {
            g.setColor(new Color(0x88, 0x88, 0xff));
        } else {
            g.setColor(new Color(0xbb, 0xbb, 0xff));
        }
        g.fillRect(0, 0, getWidth(), 20);

        g.setColor(Color.black);
        g.drawRect(0, 0, getWidth() - 1, getHeight() - 1);
    }

    public TableComponent getInstance() {
        return this;
    }

    public int getTableWidth() {
        return this.getWidth();
    }

    private void createRelationBasedOnClick() {
        PanelModel pm = (PanelModel) (getInstance().getParent());
        if (getTable().getPrimaryKey().size() != pm.getListOfSelectedAttributes().size()) {
            JOptionPane.showMessageDialog(getParent(), "Primary key size does not match.");
            observ.notifyObservers(pm.getActiveTable().getSelectedIAttribute());
            pm.setMode(0);
            setBorder(null);
            observ.notifyObservers(getSelectedIAttribute());
            pm.repaint();
            return;
        }
        boolean dataTypeMatch = true;
        for (int i = 0; i < getTable().getPrimaryKey().size(); i++) {
            if (getTable().getPrimaryKey().get(i).getDataType() != pm.getListOfSelectedAttributes().get(i).getDataType()) {
                dataTypeMatch = false;
            }
        }
        if (!dataTypeMatch) {
            JOptionPane.showMessageDialog(getParent(), "Primary key data type does not match.");
            observ.notifyObservers(pm.getActiveTable().getSelectedIAttribute());
            pm.setMode(0);
            setBorder(null);
            observ.notifyObservers(getSelectedIAttribute());
            pm.repaint();
            return;
        }
        ArrayList<IAttribute> selAttrs = new ArrayList<IAttribute>(
                ((PanelModel) getInstance().getParent()).getListOfSelectedAttributes());
        pm.getReferences().add(
                new ReferenceData(
                pm.getActiveTable().getTable(),
                selAttrs,
                getTable()));
        pm.setMode(0);
        setBorder(null);
        pm.getListOfSelectedAttributes().clear();
        pm.setActiveTableComponent(null);
        pm.repaint();
        observ.notifyObservers(null);
    }

    private MouseListener returnTableComponentMouseListener() {
        return new MouseAdapter() {

            @Override
            public void mouseClicked(MouseEvent e) {
                PanelModel pm = (PanelModel) (getInstance().getParent());
                if (pm.getMode() == 9) {
                    createRelationBasedOnClick();
                    return;
                }
                if (pm.getMode() == 8) {
                    deleteThisTable();
                    return;
                }
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                setActive(true);
                observ.notifyObservers(getInstance());
                if (e.getButton() == MouseEvent.BUTTON3 && e.getY() <= 20) {
                    tableName.setVisible(!tableName.isVisible());
                    textFieldTableName.setVisible(!textFieldTableName.isVisible());
                    if (textFieldTableName.isVisible()) {
                        textFieldTableName.setText(table.getName());
                        textFieldTableName.requestFocus();
                    }
                }
            }

            @Override
            public void mousePressed(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                if (e.getButton() == MouseEvent.BUTTON1) {
                    lastMousePressedPoint.setLocation(e.getPoint());
                    if (e.getPoint().y <= 20) {
                        scalingMode = 1;
                    } else {
                        scalingMode = 0;
                    }
                } else {
                    scalingMode = 0;
                }
                setActive(true);
                observ.notifyObservers(getInstance());
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                scalingMode = 0;
                ((PanelModel) getParent()).revalidate();
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                    setBorder(new LineBorder(Color.red, 3));
                }
            }

            @Override
            public void mouseExited(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                    setBorder(null);
                }
            }
        };
    }

    private void deleteThisTable() {
        ((PanelModel) getParent()).deleteTableComponent(this);
    }

    private MouseMotionListener returnTableComponentMouseMotionListener() {
        return new MouseMotionAdapter() {

            @Override
            public void mouseDragged(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                if (scalingMode != 1) {
                    return;
                }
                Point newPosition;
                newPosition = getLocation();
                newPosition.translate(
                        e.getX() - lastMousePressedPoint.x,
                        e.getY() - lastMousePressedPoint.y);
                if (newPosition.x < 0) {
                    newPosition.x = 0;
                }
                if (newPosition.y < 0) {
                    newPosition.y = 0;
                }
                setLocation(newPosition);
                getParent().repaint();
            }
        };
    }

    private MouseListener returnRightDownMouseListener() {
        return new MouseAdapter() {

            @Override
            public void mousePressed(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                scalingMode = 2;
                lastMousePressedPoint.setLocation(e.getPoint());
            }

            @Override
            public void mouseEntered(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                    getInstance().setBorder(new LineBorder(Color.red, 3));
                }
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                rightDownLabel.setBorder(new LineBorder(Color.black));
                rightDownLabel.repaint();
            }

            @Override
            public void mouseExited(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                    getInstance().setBorder(null);
                }
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                rightDownLabel.setBorder(null);
            }

            @Override
            public void mouseReleased(MouseEvent e) {
                scalingMode = 0;
            }
        };
    }

    private MouseMotionListener returnRightDownMouseMotionListener() {
        return new MouseMotionAdapter() {

            @Override
            public void mouseDragged(MouseEvent e) {
                if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                    return;
                }
                Point newPosition = rightDownLabel.getLocation();
                newPosition.translate(e.getX() - lastMousePressedPoint.x, e.getY() - lastMousePressedPoint.y);
                if ((newPosition.getX() > 10) && (newPosition.getY() > 10)) {
                    rightDownLabel.setLocation(newPosition);
                    setSize(
                            rightDownLabel.getX() + rightDownLabel.getWidth(),
                            rightDownLabel.getY() + rightDownLabel.getHeight());
                }
                setComponentZOrder(rightDownLabel, 0);
                sortAttributes();
                getParent().repaint();
            }
        };
    }

    private KeyListener returnTextBoxKeyListener() {
        return new KeyAdapter() {

            @Override
            public void keyPressed(KeyEvent e) {
                if (e.getKeyCode() == 10) {
                    setActive(true);
                }
            }
        };
    }

    public void addNewAttribute() {
        table.getAttributeList().add(new Attribute(
                "attr_" + attrSequence++,
                defaultTypes.getTypeList().get(8)));
        sortAttributes();
    }

    private AttributeComponent getSelectedAttribute() {
        for (int i = 0; i < getComponentCount(); i++) {
            if (getComponent(i) instanceof AttributeComponent) {
                if (((AttributeComponent) getComponent(i)).isActive()) {
                    return (AttributeComponent) getComponent(i);
                }
            }
        }
        return null;
    }

    public IAttribute getSelectedIAttribute() {
        AttributeComponent ac = getSelectedAttribute();
        if (ac != null) {
            return ac.getAttr();
        }
        return null;
    }

    public void sortAttributes() {
        IAttribute a = null;
        try {
            a = getSelectedAttribute().getAttr();
        } catch (NullPointerException e) {
        }
        removeAll();
        this.add(tableName);
        this.add(textFieldTableName);
        this.add(rightDownLabel);
        int i = 1;
        for (IAttribute attr : table.getAttributeList()) {
            AttributeComponent ac = new AttributeComponent(attr);
            add(ac);
            ac.setBounds(1, (i++) * 20 + 1, getWidth() - 1, 20);
            if (attr == a) {
                ac.setActive(true);
                repaint();
            }

        }
    }

    public boolean setSelectedAttribute(IAttribute attr) {
        boolean b = false;
        for (int i = 0; i < getComponentCount(); i++) {
            if (getComponent(i) instanceof AttributeComponent) {
                AttributeComponent ac = (AttributeComponent) getComponent(i);
                if (ac.getAttr() == attr) {
                    ac.setActive(true);
                    b = true;
                } else {
                    ac.setActive(false);
                }
                if (ac.getTextFieldAttrName().isVisible()) {
                    ac.setAttrNameByTextBox();
                    ac.getAttrName().setVisible(true);
                    ac.getTextFieldAttrName().setVisible(false);
                }
            }
        }
        return b;
    }

    void setSelectedAttributeAs(int i, boolean select) {
        switch (i) {
            case 1:
                getSelectedAttribute().getAttr().setPrimaryKey(select);
                break;
            case 2:
                getSelectedAttribute().getAttr().setUnique(select);
                break;
            case 3:
                getSelectedAttribute().getAttr().setNotNull(select);
                break;
            case 4:
                getSelectedAttribute().getAttr().setAutoIncrement(select);
                break;
        }
        sortAttributes();
        observ.notifyObservers(getSelectedAttribute().getAttr());
    }

    void setCurrentAttributeType(ISqlDataType iSqlDataType) {
        getSelectedAttribute().getAttr().setDataType(iSqlDataType);
        sortAttributes();
        observ.notifyObservers(getSelectedAttribute().getAttr());
    }

    void setCurrentAttributeArgument1(Integer integer) {
        IAttribute attr = getSelectedIAttribute();
        if (attr == null) {
            return;
        }
        if (attr.getArguments().isEmpty()) {
            return;
        }
        attr.getArguments().set(0, new Integer(integer.intValue()));
        sortAttributes();
        observ.notifyObservers(getSelectedAttribute().getAttr());
    }

    void setCurrentAttributeArgument2(Integer integer) {
        IAttribute attr = getSelectedIAttribute();
        if (attr == null) {
            return;
        }
        if (attr.getArguments().size() < 2) {
            return;
        }
        getSelectedAttribute().getAttr().getArguments().set(1, new Integer(integer.intValue()));
        sortAttributes();
        observ.notifyObservers(getSelectedAttribute().getAttr());
    }

    void deleteCurrentAttribute() {
        table.getAttributeList().remove(getSelectedAttribute().getAttr());
        sortAttributes();
        observ.notifyObservers(getInstance());
    }

    void moveCurrentAttributeUp() {
        AttributeComponent ac = getSelectedAttribute();
        if (ac == null) {
            return;
        }
        int i = table.getAttributeList().indexOf(getSelectedAttribute().attr);
        if (i > 0) {
            table.getAttributeList().add(i - 1, table.getAttributeList().get(i));
            table.getAttributeList().remove(i + 1);
        }
        sortAttributes();
        observ.notifyObservers(ac.getAttr());
    }

    void moveCurrentAttributeDown() {
        AttributeComponent ac = getSelectedAttribute();
        if (ac == null) {
            return;
        }
        int i = table.getAttributeList().indexOf(getSelectedAttribute().attr);
        if (i < table.getAttributeList().size() - 1) {
            table.getAttributeList().add(i + 2, table.getAttributeList().get(i));
            table.getAttributeList().remove(i);
        }
        sortAttributes();
        observ.notifyObservers(ac.getAttr());
    }

    void setSelectedAttributes(ArrayList<IAttribute> selectedAttributes) {
        for (int i = 0; i < getComponentCount(); i++) {
            if (getComponent(i) instanceof AttributeComponent) {
                AttributeComponent ac = (AttributeComponent) getComponent(i);
                if (selectedAttributes.indexOf(ac.getAttr()) != -1) {
                    ac.setActive(true);
                } else {
                    ac.setActive(false);
                }
                if (ac.getTextFieldAttrName().isVisible()) {
                    ac.setAttrNameByTextBox();
                    ac.getAttrName().setVisible(true);
                    ac.getTextFieldAttrName().setVisible(false);
                }
            }
        }
    }

    private AttributeComponent getAttrComponent(IAttribute attr) {
        for (int i = 0; i < getComponentCount(); i++) {
            if (getComponent(i) instanceof AttributeComponent) {
                if (((AttributeComponent) getComponent(i)).getAttr() == attr) {
                    return (AttributeComponent) getComponent(i);
                }
            }
        }
        return null;
    }

    void showFKComponents(ArrayList<IAttribute> attributeWithPK, boolean show) {
        for (IAttribute iAttribute : attributeWithPK) {
            getAttrComponent(iAttribute).setShowRelation(show);
        }
    }

    private class AttributeComponent extends JComponent {

        private JLabel attrName = new JLabel();
        private JTextField textFieldAttrName = new JTextField();
        private JLabel attrType = new JLabel();
        private IAttribute attr;
        private boolean active;
        private boolean hover = false;
        private boolean showRelation = false;

        public AttributeComponent(IAttribute attr) {
            this.attr = attr;
            this.attrName.setText(attr.getName());
            add(attrName);
            attrName.setBounds(25, 0, 120, 20);
            add(textFieldAttrName);
            textFieldAttrName.setBounds(20, 0, 120, 20);
            textFieldAttrName.setVisible(false);
            StringBuilder attrTypeText = new StringBuilder(attr.getDataType().getShortCut());
            if (attr.getDataType().getArgs().size() > 0) {
                attrTypeText.append("(");
                for (int i = 0; i < attr.getDataType().getArgs().size(); i++) {
                    attrTypeText.append(attr.getArguments().get(i));
                    if (i != attr.getDataType().getArgs().size() - 1) {
                        attrTypeText.append(", ");
                    }
                }
                attrTypeText.append(")");
            }
            attrType.setText(attrTypeText.toString());
            add(attrType);
            attrType.setBounds(146, 0, 120, 20);

            addMouseListener(returnAttributeComponentMouseListener());
            attrName.addMouseListener(returnAttrNameMouseListener());
            textFieldAttrName.addKeyListener(returnAttrTextBoxKeyListener());
        }

        public boolean isActive() {
            return active;
        }

        public void setActive(boolean active) {
            this.active = active;
        }

        public JLabel getAttrName() {
            return attrName;
        }

        public void setAttrName(JLabel attrName) {
            this.attrName = attrName;
        }

        public JTextField getTextFieldAttrName() {
            return textFieldAttrName;
        }

        public void setTextFieldAttrName(JTextField textFieldAttrName) {
            this.textFieldAttrName = textFieldAttrName;
        }

        public IAttribute getAttr() {
            return attr;
        }

        public void setAttr(IAttribute attr) {
            this.attr = attr;
        }

        public boolean isShowRelation() {
            return showRelation;
        }

        public void setShowRelation(boolean showRelation) {
            this.showRelation = showRelation;
        }

        @Override
        protected void paintComponent(Graphics g) {
            super.paintComponent(g);

            if (hover) {
                g.setColor(new Color(0xcc, 0xff, 0xff));
                g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
            }
            if (active) {
                g.setColor(new Color(0x88, 0xff, 0xff));
                g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
            }
            if (attr.isAutoIncrement()) {
                g.drawImage(icons[0].getImage(), 0, 0, this);
            }
            if (attr.isNotNull() && !attr.isUnique() && !attr.isPrimaryKey()) {
                g.drawImage(icons[1].getImage(), 0, 0, this);
            }
            if (attr.isUnique() && !attr.isPrimaryKey()) {
                g.drawImage(icons[2].getImage(), 0, 0, this);
            }
            if (attr.isPrimaryKey()) {
                g.drawImage(icons[3].getImage(), 0, 0, this);
            }
            if (!attr.getForeignKey().isEmpty()) {
                g.drawImage(icons[4].getImage(), 0, 0, this);
            }

            if (showRelation) {
                g.setColor(new Color(0xff, 0, 0, 0x88));
                g.fillRect(0, 0, getWidth() - 1, getHeight() - 1);
            }

        }

        private AttributeComponent returnAttributeComponentInstance() {
            return this;
        }

        private MouseListener returnAttributeComponentMouseListener() {
            return new MouseAdapter() {

                @Override
                public void mouseClicked(MouseEvent e) {
                    PanelModel pm = (PanelModel) (getInstance().getParent());
                    if (pm.getMode() == 9) {
                        createRelationBasedOnClick();
                        return;
                    }
                    if (pm.getMode() == 8) {
                        deleteThisTable();
                        return;
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                        return;
                    }
                    if (e.getButton() == MouseEvent.BUTTON1) {
                        if ((e.getModifiersEx() & MouseEvent.CTRL_DOWN_MASK) == MouseEvent.CTRL_DOWN_MASK) {
                            if ((getSelectedIAttribute() != null)) {
                                if (((PanelModel) getInstance().getParent()).getListOfSelectedAttributes().
                                        indexOf(returnAttributeComponentInstance().getAttr()) == -1) {
                                    ((PanelModel) getInstance().getParent()).setCanSelectMoreAttributes(true);
                                }
                            }
                        } else {
                            ((PanelModel) getInstance().getParent()).getListOfSelectedAttributes().clear();
                        }
                        active = true;
                        observ.notifyObservers(returnAttributeComponentInstance().getAttr());
                        repaint();
                    }
                }

                @Override
                public void mouseEntered(MouseEvent e) {
                    if (scalingMode != 0) {
                        return;
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                        getInstance().setBorder(new LineBorder(Color.red, 3));
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                        return;
                    }
                    if (!hover) {
                        hover = true;
                        if (!attr.getForeignKey().isEmpty()) {
                            ((PanelModel) (getInstance().getParent())).showRelationShips(attr.getForeignKey(), true);
                        }
                        repaint();
                    }
                }

                @Override
                public void mouseExited(MouseEvent e) {
                    if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                        getInstance().setBorder(null);
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                        return;
                    }
                    if (hover) {
                        hover = false;
                        if (!attr.getForeignKey().isEmpty()) {
                            ((PanelModel) (getInstance().getParent())).showRelationShips(attr.getForeignKey(), false);
                        }
                        repaint();
                    }
                }
            };
        }

        private MouseListener returnAttrNameMouseListener() {
            return new MouseAdapter() {

                @Override
                public void mouseClicked(MouseEvent e) {
                    PanelModel pm = (PanelModel) (getInstance().getParent());
                    if (pm.getMode() == 9) {
                        createRelationBasedOnClick();
                        return;
                    }
                    if (pm.getMode() == 8) {
                        deleteThisTable();
                        return;
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                        return;
                    }
                    if (e.getButton() == MouseEvent.BUTTON3) {
                        observ.notifyObservers(returnAttributeComponentInstance().getAttr());
                        active = true;
                        setSelectedAttribute(attr);
                        getInstance().repaint();
                        textFieldAttrName.setText(attr.getName());
                        attrName.setVisible(false);
                        textFieldAttrName.setVisible(true);
                        textFieldAttrName.requestFocus();
                    }
                    if (e.getButton() == MouseEvent.BUTTON1) {
                        if ((e.getModifiersEx() & MouseEvent.CTRL_DOWN_MASK) == MouseEvent.CTRL_DOWN_MASK) {
                            if (getSelectedIAttribute() != null) {
                                if (((PanelModel) getInstance().getParent()).getListOfSelectedAttributes().
                                        indexOf(returnAttributeComponentInstance().getAttr()) == -1) {
                                    ((PanelModel) getInstance().getParent()).setCanSelectMoreAttributes(true);
                                }
                            }
                        }
                        active = true;
                        observ.notifyObservers(returnAttributeComponentInstance().getAttr());
                        repaint();
                    }
                }

                @Override
                public void mouseEntered(MouseEvent e) {
                    if (scalingMode != 0) {
                        return;
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                        getInstance().setBorder(new LineBorder(Color.red, 3));
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                        return;
                    }
                    if (!hover) {
                        hover = true;
                        if (!attr.getForeignKey().isEmpty()) {
                            ((PanelModel) (getInstance().getParent())).showRelationShips(attr.getForeignKey(), true);
                        }
                        repaint();
                    }
                }

                @Override
                public void mouseExited(MouseEvent e) {
                    if (((PanelModel) getInstance().getParent()).getMode() >= 8) {
                        getInstance().setBorder(null);
                    }
                    if (((PanelModel) getInstance().getParent()).getMode() != 0) {
                        return;
                    }
                    if (hover) {
                        hover = false;
                        if (!attr.getForeignKey().isEmpty()) {
                            ((PanelModel) (getInstance().getParent())).showRelationShips(attr.getForeignKey(), false);
                        }
                        repaint();
                    }
                }
            };
        }

        public void setAttrNameByTextBox() {
            if (textFieldAttrName.getText().contains(" ")) {
                return;
            }
            attrName.setText(textFieldAttrName.getText());
            attr.setName(textFieldAttrName.getText());
        }

        private KeyListener returnAttrTextBoxKeyListener() {
            return new KeyAdapter() {

                @Override
                public void keyPressed(KeyEvent e) {
                    if (e.getKeyCode() == 10) {
                        setAttrNameByTextBox();
                        sortAttributes();
                    }
                }
            };
        }

        @Override
        public Dimension getPreferredSize() {
            return new Dimension(getTableWidth(), 20);
        }
    }

    private class Observ extends Observable {

        @Override
        public void notifyObservers(Object arg) {
            setChanged();
            super.notifyObservers(arg);
        }
    }
}
